# /user/bin/env python3
# -*- coding: utf-8 -*-
"""All exceptions of module."""


class LowCovError(Exception):
    """Low reads coverage."""

    pass


class GeneFormatError(Exception):
    """Gene in wrong format thus cannot be read."""

    pass


class WrongConfigFormatError(Exception):
    """Wrong config format which cannot be read."""

    pass


class ConfigNotSetError(Exception):
    """Config file not set error."""

    pass


class GenesNotSetError(Exception):
    """Genes not set for sample class error."""

    pass


class InfoNotSetError(Exception):
    """Info not set for sample class error."""

    pass


class PropMismatchError(Exception):
    """Mismatching properties found within input xlsx file."""

    pass


class MissingSampleInfoFileError(Exception):
    """Sample info file cannot be found error."""

    pass


class MissingLabInfoFileError(Exception):
    """Lab info file cannot be found error."""

    pass


class MissingMergedInfoFileError(Exception):
    """Merged info file cannot be found error."""

    pass


class SampleInfoFormatError(Exception):
    """Sample info in wrong format thus cannot be read."""

    pass


class NoneConfigedTableError(Exception):
    """Table not found in config file."""

    pass


class WrongTableColumnCountError(Exception):
    """Table not having correct number of column."""

    pass


class WrongTableRowCountError(Exception):
    """Table not having correct number of row."""

    pass


class WrongTableContentColumnCountError(Exception):
    """Table row content not having correct number of column."""

    pass


class NoKdbRecordFoundError(Exception):
    """No kdb record can be found error."""

    pass


class InfoFileNotFoundError(Exception):
    """Info file cannot be found."""

    pass


class WrongKdbFormatError(Exception):
    """Wrong kdb format."""

    pass


class MissingResidentialIdError(Exception):
    """Missing residential id for yuedong report."""

    pass


class MissingYueDongRecordError(Exception):
    """YueDong sample with missing previous record in database."""

    pass


class DuplicatedPcError(Exception):
    """Multiple positive control found within input."""

    pass


class NoPcError(Exception):
    """No positive control found within input."""

    pass
